/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;
import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class MandatoryFeature here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: Choice.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class Choice extends FodaPA {
    public static final String NAME = "choose_1";
    /**
     * Creates a new <code>MandatoryFeature</code> instance.
     *
     */
    public Choice(List<FodaPA> terms) {
	super();
	setSubterms(terms);
	size = 1+FodaPA.maxSize(this.getSubterms());
    }


    // private static IVec<IVecInt> makeDisjoint(IVec<IVecInt> vec1, IVec<IVecInt> vec2) {
    // 	IVec<IVecInt> newVec = new Vec<IVecInt>();
    // 	log.debug(vec1.size()+":"+vec2.size());
    // 	for (Iterator<IVecInt> i = vec1.iterator(); i.hasNext();) {
    // 	    IVecInt vecInt_i = i.next();
    // 	    for (Iterator<IVecInt> j = vec2.iterator(); j.hasNext();) {
    // 		IVecInt vecInt_j = j.next();
    // 		IVecInt newVecInt = new VecInt();
    // 		vecInt_i.copyTo(newVecInt);
    // 		vecInt_j.copyTo(newVecInt);
    // 		newVec.push(newVecInt);
    // 	    }
    // 	}
    // 	return newVec;
    // } 
    private static int genNewVar(List<Integer> newVarLst, LiteralMap idMap) {
	int newVar = idMap.size()+1;
    	String newVarName = AUX_VAR_PREFIX+"ch"+newVar;
	int literal = idMap.getLiteral(newVarName);
	newVarLst.add(literal);
	return literal;
    }
    private static void insertNewClauses(IVec<IVecInt> newClauses, IVec<IVecInt> clauses,
					 int newVar, List<Integer> newVarLst) {
	for (int i = 0;  i<clauses.size() ; i++) {
	    IVecInt clause = clauses.get(i);
	    IVecInt newClause = new VecInt();
	    clause.copyTo(newClause);
	    for (Integer var: newVarLst) {
		if (var.intValue()==newVar) {
		    newClause.push(-newVar);
		} else {
		    newClause.push(var.intValue());
		}
	    }
	    newClauses.push(newClause);
	}
    }
    public final IVec<IVecInt> getClauses(LiteralMap idMap,
					  boolean full) {
	IVec<IVecInt> newClauses = new Vec<IVecInt>();
	/* Instead of making all possible disjoints
	   we use the following property P or Q is equivalent to Z->Q and not Z -> P
	   where Z is a new variable */
	LinkedList<Integer> newVarLst = new LinkedList<Integer>();
	for (FodaPA term: this.getSubterms()) {
	    int newVar = genNewVar(newVarLst, idMap);
	    IVec<IVecInt> clauses = term.getClauses(idMap, full);
	    insertNewClauses(newClauses, clauses, newVar, newVarLst); 
	}
	/* We have to include a new clause nor Z->T, where Z is the last
	   variable */
	Integer lastVar = newVarLst.getLast();
	IVecInt clause = new VecInt();
	clause.push(lastVar.intValue());
	newClauses.push(clause);
	return newClauses;
    }
    // public void addClauses(ISolver solver, HashMap<String,Integer> idMap,
    // 				    boolean full ) 
    // 	throws ContradictionException {
    // 	solver.addAllClauses(this.getClauses(idMap,full));
    // }

    
    /**
     * Describe <code>reduce</code> method here.
     *
     * @return a <code>ComboFodaPABool</code> value
     */
    public final FodaPA reduce() {
	boolean hasReduced = false;
	
	int nr = numReductions;
    	List<FodaPA> newSubterms = new LinkedList<FodaPA>();
	for (FodaPA term: this.getSubterms()) {
	    FodaPA newTerm = term.reduce();
	    if (newTerm.isNil()) {
		numReductions++; // this term is not added so it counts as a reduction.
	    } else {
		newSubterms.add(newTerm);
	    }
	}
	
	FodaPA fodaPA = null;
	if (newSubterms.size()==0) {
	    numReductions++;
	    fodaPA = new Nil();
	} else if (newSubterms.size()==1) {
	    numReductions++;
	    fodaPA = newSubterms.get(0);
	} else {
	    //removeSmallRepeated(newSubterms);
	    fodaPA = new Choice(newSubterms);
	}
	return fodaPA;
    }
      
    private static final int SMALL_THRESHOLD = 3;
    private void removeSmallRepeated(List<FodaPA> terms) {
    	for (ListIterator<FodaPA> iter  = terms.listIterator(); iter.hasNext();) {
	    FodaPA term = iter.next();
	    if (term.size<=SMALL_THRESHOLD) {
		int repeated = 0;
		int next = iter.nextIndex();
		for (Iterator<FodaPA> iter2  = terms.listIterator(next); iter2.hasNext();) {
		    FodaPA term2 = iter2.next();
		    if (term!=term2 && term.equals(term2)) {
			repeated++;
		    }
		}
		if (repeated>1) {
		    numReductions++;
		    FodaPA.log.debug("lo conseguí!");
		    iter.remove();
		}
	    }
	}
    }
    public String toString() {
	return "("+StringUtils.join(this.getSubterms(), ") [] (")+")";
    }

    public static Set<Product> denotational(List<Set<Product>> subterms) {
	Set<Product> s = new HashSet<Product>();
	for (Set<Product> sp : subterms) {
	    s.addAll(sp);
	}
	return s;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	Set<Product> s = new HashSet<Product>();
	for (Iterator<FodaPA> itr = getSubterms().iterator(); itr.hasNext();) {
	    Set<Product> sp = itr.next().denotational();
	    s.addAll(sp);
	}
	return s;
    }
    
}
