/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */

package fodaat;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import java.util.Date;
public class Denotational {

    private static List<Set<Product>> getSubTerms(Element element) {
	List<Set<Product>> subterms = new LinkedList<Set<Product>>();
	NodeList lst = element.getChildNodes();
	
	for (int i = 0; i < lst.getLength(); i++) {
	    if ( lst.item(i) instanceof Element ) {
		subterms.add(buildTerm((Element)lst.item(i)));
	    }
	}
	
	return subterms;
    }


    private static Set<Product> buildTerm(Element element) {
	String elementName = element.getNodeName();
	if (elementName.equals(Nil.NAME)) {
	    return new Nil().denotational();
	} else if (elementName.equals(Checkmark.NAME)) {
	    return new Checkmark().denotational();
	} else if (elementName.equals(Prefix.NAME)) {
	    String feature = element.getAttribute("name");
	    return Prefix.denotational(feature, getSubTerms(element).get(0));
	} else if (elementName.equals(OptionalPrefix.NAME)) {
	    String feature = element.getAttribute("name");
	    return OptionalPrefix.denotational(feature, 
					       getSubTerms(element).get(0));
	} else if (elementName.equals(Choice.NAME)) {
	    Set<Product> s =  Choice.denotational(getSubTerms(element));
	    System.gc();
	    return s;
	} else if (elementName.equals(Parallel.NAME)) {
	    Set<Product> s = Parallel.denotational(getSubTerms(element));
	    System.gc();
	    return s;
	} else if (elementName.equals(Requires.NAME)) {
	    String featPre = element.getAttribute("feature_1");
	    String featSeq = element.getAttribute("feature_2");
	    Set<Product> subterm = getSubTerms(element).get(0);
	    return Requires.denotational(featPre, featSeq, subterm);
	} else if (elementName.equals(Excludes.NAME)) {
	    String featPre = element.getAttribute("feature_1");
	    String featSeq = element.getAttribute("feature_2");
	    Set<Product> subterm = getSubTerms(element).get(0);
	    return Excludes.denotational(featPre, featSeq, subterm);
	} else if (elementName.equals(Mandatory.NAME)) {
	    String feature = element.getAttribute("name");
	    return Mandatory.denotational(feature, getSubTerms(element).get(0));
	} else if (elementName.equals(Forbid.NAME)) {
	    String feature = element.getAttribute("name");
	    return Forbid.denotational(feature, getSubTerms(element).get(0));
	}
	return null;
    }
    
   
    protected static Set<Product> readXML(File file) 
	throws ParserConfigurationException, SAXException, IOException {
	DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
	DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
	Document doc = dBuilder.parse(file);
	doc.getDocumentElement().normalize();
	NodeList lst  = (doc.getChildNodes().item(0).getChildNodes());
	for (int i = 0; i<lst.getLength() ; i++) { //Only one child of the root element is an elements
	    //and it is the top level term.
	    if ( lst.item(i) instanceof Element ) {
		return buildTerm((Element)lst.item(i));
	    }
	}
	return null;
    }
    public static void main(String [] args) throws Exception {
	File file = new File(args[0]);
	boolean full = false;
	
	Date start = new Date();
	System.out.print(args[0]);
	Set<Product> products = readXML(file);
	Date end = new Date();
	System.out.println(String.format(" %d %d %s",end.getTime()-start.getTime(), products.size(), products));
	
	//System.out.println();
	// start = new Date();
	// solve(fodaPA, true);
	// end = new Date();
	// System.out.print(end.getTime()-start.getTime());
    }        
}
