/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Describe class FodaATAxiom here.
 *
 *
 * Created: Sun Nov 25 19:07:00 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: FodaPA.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public abstract class FodaPA implements Cloneable {
    protected static final String AUX_VAR_PREFIX="_aux";
    public static Logger log = Logger.getLogger("fodat");
    protected static int numReductions=0;
    private static boolean axStrict;
    
    public static void setAxStrict(boolean b) {
	axStrict = b;
    }
    public static boolean getAxStrict() {
	return axStrict;
    }
    
    protected int size;
    private List<String> features;
    private List<FodaPA> subterms;
    public FodaPA() {
	size=0;
	features = null;
	subterms = null;
    }
    public FodaPA clone() {	
	try {
	    FodaPA newTerm = (FodaPA)super.clone();
	    if (this.subterms!=null) {
		newTerm.subterms = new LinkedList<FodaPA>();
		for (FodaPA subterm: subterms) {
		    newTerm.subterms.add(subterm.clone());
		}
	    }
	    // log.debug(newTerm.getClass()+":"+(subterms==newTerm.subterms)+
	    // 	      ":"+(subterms!=null && subterms.size()>=1 ? subterms.get(0)==newTerm.subterms.get(0):"No" ));
	    return newTerm;
	} catch (CloneNotSupportedException e) {
	    return null;
	}

    }
    public List<String> getFeatures() {
	return this.features;
    }
    public void addFeatures(LiteralMap idMap) {
	if (this.getFeatures()!=null) {
	    for (String feature: this.features) {
		int lit = idMap.getLiteral(feature, false);
	    }
	}
	if (this.getSubterms()!=null) {
	    for (FodaPA term: this.subterms) {
		term.addFeatures(idMap);
	    }
	}

    }
    
    public void setFeature(String feature){
	if (this.features==null) {
	    this.features = new LinkedList<String>();
	    this.features.add(null);
	}
	this.features.set(0,feature);
    }
    public String getFeature() {
	return this.features.get(0);
    }
    
    public void setFeatPre(String feature) {
	setFeature(feature);
    }
    public void setFeatSeq(String feature) {
	if (this.features==null) {
	    this.features = new LinkedList<String>();
	    this.features.add(null);
	    this.features.add(null);
	}
	if (this.features.size()==1) {
	    this.features.add(null);
	}
	this.features.set(1,feature);
    }
    
    public String getFeatPre() {
	return getFeature();
    }
    public String getFeatSeq() {
	return this.features.get(1);
    }
    public void setSubterm(FodaPA subterm) {
	if (this.subterms == null) {
	    this.subterms = new LinkedList<FodaPA>();
	    this.subterms.add(null);
	}
	this.subterms.set(0,subterm);
    }
    public FodaPA getSubterm() {
	return this.subterms.get(0);
    }
    public void setSubterms(List<FodaPA> subterms) {
	this.subterms = subterms;
    }
    public List<FodaPA> getSubterms() {
	return this.subterms;
    }
    
	
    public int getSize(){
	return size;
    }
    
    protected static final boolean equalsTermsLST(List<FodaPA> terms1,
						  List<FodaPA> terms2) {
	
	if (terms2.size()!=terms1.size()) {
	    return false;
	}

	return terms1.containsAll(terms2) && terms2.containsAll(terms1);
	// for (Iterator<FodaPA> itr = terms1.iterator(); itr.hasNext();) {
	//     FodaPA term = itr.next();
	//     if (!terms2.contains(term)) {
	// 	return false;
	//     }
	// }
	// for (Iterator<FodaPA> itr = terms2.iterator(); itr.hasNext();) {
	//     FodaPA term = itr.next();
	//     if (!terms1.contains(term)) {
	// 	return false;
	//     }
	// }
	// return true;
    }
    
    // public boolean equals(Object other) {
    // 	if ( !( other instanceof FodaPA ) ) {
    // 	    return false;
    // 	} 
    // 	FodaPA fother = (FodaPA) other;
    // 	return 
    // 	    this.getFeatures().equals(fother.getFeatures()) &&
    // 	    equalsTermsLST(this.getSubterms(),  fother.getSubterms());
    // }
    protected static final int maxSize(List<FodaPA> terms){
	int max = 0;
	for (Iterator<FodaPA> iter = terms.iterator(); iter.hasNext();) {
	    FodaPA term = iter.next();
	    if (term.size>max) {
		max = term.size;
	    }
	}
	return max;
    }
    private static List<FodaPA> getSubTerms(Element element) {
	List<FodaPA> subterms = new LinkedList<FodaPA>();
	NodeList lst = element.getChildNodes();
	
	for (int i = 0; i < lst.getLength(); i++) {
	    if ( lst.item(i) instanceof Element ) {
		subterms.add(buildTerm((Element)lst.item(i)));
	    }
	}
	
	return subterms;
    }
        
    
    
    private static FodaPA buildTerm(Element element) {
	String elementName = element.getNodeName();
	if (elementName.equals(Nil.NAME)) {
	    return new Nil();
	} else if (elementName.equals(Checkmark.NAME)) {
	    return new Checkmark();
	} else if (elementName.equals(Prefix.NAME)) {
	    String feature = element.getAttribute("name");
	    return new Prefix(feature, getSubTerms(element).get(0));
	} else if (elementName.equals(OptionalPrefix.NAME)) {
	    String feature = element.getAttribute("name");
	    FodaPA term = new OptionalPrefix(feature, 
					     getSubTerms(element).get(0));
	    return term;
	} else if (elementName.equals(Choice.NAME)) {
	    return new Choice(getSubTerms(element));
	} else if (elementName.equals(Parallel.NAME)) {
	    return new Parallel(getSubTerms(element));
	} else if (elementName.equals(Requires.NAME)) {
	    String featPre = element.getAttribute("feature_1");
	    String featSeq = element.getAttribute("feature_2");
	    FodaPA subterm = getSubTerms(element).get(0);
	    return new Requires(featPre, featSeq, subterm);
	} else if (elementName.equals(Excludes.NAME)) {
	    String featPre = element.getAttribute("feature_1");
	    String featSeq = element.getAttribute("feature_2");
	    FodaPA subterm = getSubTerms(element).get(0);
	    return new Excludes(featPre, featSeq, subterm);
	} else if (elementName.equals(Mandatory.NAME)) {
	    String feature = element.getAttribute("name");
	    return new Mandatory(feature, getSubTerms(element).get(0));
	} else if (elementName.equals(Forbid.NAME)) {
	    String feature = element.getAttribute("name");
	    return new Forbid(feature, getSubTerms(element).get(0));
	}
	return null;
    }
    
   
    protected static FodaPA readXML(File file) 
	throws ParserConfigurationException, SAXException, IOException {
	DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
	DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
	Document doc = dBuilder.parse(file);
	doc.getDocumentElement().normalize();
	NodeList lst  = (doc.getChildNodes().item(0).getChildNodes());
	for (int i = 0; i<lst.getLength() ; i++) { //Only one child of the root element is an elements
	    //and it is the top level term.
	    if ( lst.item(i) instanceof Element ) {
		return buildTerm((Element)lst.item(i));
	    }
	}
	return null;
    }
    
    public boolean isNil() {
	return false;
    }
    
    public abstract FodaPA reduce();
    public abstract Set<Product> denotational();
    
    public FodaPA reduceAll() {
	boolean cont = true;
	FodaPA term = this;
	while (cont) {
	    // log.debug(term);
	    int nr = numReductions;
	    term = term.reduce();
	    cont = nr!=numReductions;
	}
	// log.debug(term);
	// log.debug(numReductions);
	return term;
    }
    
    private boolean contains(FodaPA term) {
	if (this==term) {
	    return true;
	}
	if (this.subterms==null) {
	    return false;
	}
	for (FodaPA subterm: this.subterms) {
	    if (subterm.contains(term)) {
		return true;
	    }
	}
	return false;
    }

    
    
    public static void substitueCheckmark(FodaPA term, FodaPA subs) {
	// term is not checkmark or nil
	for (int i = 0;  i<term.getSubterms().size(); i++) {
	    FodaPA current = term.getSubterms().get(i);
	    // log.debug(term+":"+term.getClass()+":\n"+current+":"+i+":"+subs+":"+
	    // 	      ((subs.subterms!=null && subs.subterms.size()>=1 &&
	    // 		subs.subterms.get(0).subterms!=null &&  subs.subterms.get(0).subterms.size()>=1) ? 
	    // 	       subs.subterms.get(0).subterms.get(0)+":"+(subs.subterms.get(0).subterms.get(0)==term):"No"));		      
	    if (current instanceof Checkmark ) {
		numReductions++;
		term.getSubterms().set(i, subs.clone());
	    } else if ( !(current instanceof Nil) ) {
		// log.debug(current+":"+subs+":"+(
		// 				(subs.subterms!=null && subs.subterms.size()>=1)?""+(subs.subterms.get(0)==current):"NO:"));
		substitueCheckmark(current, subs);
	    } 
	}
    }
	
    public abstract IVec<IVecInt> getClauses(LiteralMap idMap,
					     boolean full);
           
    
    
    protected final static void addToAll(IVec<IVecInt> clauses, int literal) {
	for (Iterator<IVecInt> itr = clauses.iterator(); itr.hasNext();) {
	    IVecInt clause =itr.next();
	    clause.push(literal);
	}
    }
}

// class ComboFodaPABool {
//     FodaPA term;
//     boolean hasBeenReduced;
//     ComboFodaPABool(FodaPA term, boolean hasBeenReduced) {
// 	this.term = term;
// 	this.hasBeenReduced = hasBeenReduced;
//     } 
    
//     FodaPA getTerm() {
// 	return term;
//     }
    
//     boolean getHasBeenReduced () {
// 	return hasBeenReduced;
//     }
//     public String toString() {
// 	return String.format("%1s:%2s",term,hasBeenReduced);
//     }
//}