/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class Forbid here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: Forbid.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class Forbid extends FodaPA {
    public static final String NAME = "forbid";
    /**
     * Creates a new <code>Forbid</code> instance.
     *
     */
    public Forbid(String feature, FodaPA term) {
	setSubterm(term);
	setFeature(feature);
	this.size = this.getSubterm().size + 1;
    }



    /**
     * Describe <code>reduce</code> method here.
     *
     * @return a <code>ComboFodaPABool</code> value
     */
    public final FodaPA reduce() {
	int nr=numReductions;
	FodaPA subterm = this.getSubterm();
	if (subterm.isNil()) {
	    numReductions++;
	    return subterm;
	} else if (subterm instanceof Checkmark) {
	    numReductions++;
	    return subterm;
	} else if (subterm instanceof Prefix) {
	    Prefix prefix = (Prefix)subterm;
	    if (prefix.getFeature().equals(this.getFeature())) {
		numReductions++;
		return new Nil();
	    } else {
		numReductions++;
		return new Prefix(prefix.getFeature(), 
				  new Forbid(this.getFeature(),prefix.getSubterm()));
	    }
	} else if (subterm instanceof Choice) {
	    List<FodaPA> subterms = new LinkedList<FodaPA>();
	    for (FodaPA term: ((Choice)subterm).getSubterms()) {
		numReductions++;
		subterms.add(new Forbid(this.getFeature(), term));
	    }
	    return new Choice(subterms);
	} else {
	FodaPA newSubterm = subterm.reduce();
	    if (nr<numReductions) {
		return new Forbid(this.getFeature(), newSubterm);
	    } else {
		return this;
	    }
	}
    }

    public final IVec<IVecInt> getClauses(LiteralMap idMap, 
					  boolean full) {
	IVec<IVecInt> clauses = this.getSubterm().getClauses(idMap, full);
	int literal = idMap.getLiteral(this.getFeature(),true);
	int [] clause = {-literal};
	IVecInt vecInt = new VecInt(clause);
	clauses.push(vecInt);
	return clauses;
    }
    
    // public void addClauses(ISolver solver, HashMap<String,Integer> idMap, 
    // 			   boolean full) 
    // 	throws ContradictionException{
    // 	int literal = FodaPA.getLiteral(idMap, this.getFeature());
    // 	int [] clause = {-literal};
    // 	IVecInt vecInt = new VecInt(clause);
    // 	solver.addClause(vecInt);
    // 	thisthis.getSubterm().addClauses(solver, idMap, full);
    // }
    
    public static Set<Product> denotational(String feat, 
					   Set<Product> s) {
	for (Iterator<Product> itr = s.iterator(); itr.hasNext();) {
	    Product p = itr.next();
	    if ( p.contains(feat) )  {
		itr.remove();
	    }
	}
	return s;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	return denotational(getFeature(), 
			    getSubterm().denotational());
    }

     
    public String toString() {
    	return String.format("(%1s)\\%2s",this.getSubterm().toString(),this.getFeature());
    }
}
