/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.HashMap;
import java.util.Vector;
import org.sat4j.core.VecInt;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * This class is used to map identifiers in foda terms with
   literals to de sat solver. 
   
   Each identifier can occour in multiple restrictions. 
   Each ocurrence of a variable in different restrictions must be marked
   as different. Therefore each identifier has a vector of different ocurrences
   in a restriction.
 *
 *
 * Created: Mon Dec  3 14:46:18 2012
 *
 * @author <a href="mailto:luis@kimba.mat.ucm.es"></a>
 * @version $Id: LiteralMap.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class LiteralMap extends HashMap<String, Vector<Integer>> {

    /**
     * Creates a new <code>LiteralMap</code> instance.
     *
     */
    private int numLiterals=0;
    public LiteralMap() {
	super();
	numLiterals = 0;
    }

    public int size() {
	return numLiterals;
    }    
    public final int getLiteral(String feature,
				boolean isRestriction) {
	Vector<Integer> litVector = this.get(feature);
	if (litVector==null) {
	    litVector = new Vector<Integer>();
	    this.put(feature, litVector);
	}
	if (litVector.size()==0 || isRestriction) {
	    numLiterals++;
	    litVector.add(new Integer(numLiterals++));
	}
	return litVector.lastElement().intValue();
    }
    
    public final int getLiteral(String feature) {
	return this.getLiteral(feature, false);
    }
    
    public final void addSequentialRestrictions(IVec<IVecInt> clauses) {
	for (String feature : this.keySet()) {
	    Vector<Integer> litVector = this.get(feature);
	    for (int i = 0;  i<litVector.size()-1 ; i++) {
		int [] clause = {litVector.get(i+1), -litVector.get(i)};
		clauses.push(new VecInt(clause));
	    }
	}
    }
    
    
    public String toString() {
	StringBuffer sb = new StringBuffer();
	for (String feature: this.keySet()) {
	    sb.append(feature+":");
	    Vector<Integer> litVector = this.get(feature);
	    for (int i = 0;  i<litVector.size() ; i++) {
		sb.append(String.format("(%d,%d)",i,litVector.get(i)));
	    }
	    sb.append("\n");
	}
	return sb.toString();
    }
}
