/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.sat4j.core.Vec;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class MandatoryFeature here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: OptionalPrefix.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class OptionalPrefix extends FodaPA {
    public static final String NAME = "optional_feature";
    /**
     * Creates a new <code>Prefix</code> instance.
     *
     */
    
    public OptionalPrefix(String feature, FodaPA term) {
	super();
	setFeature(feature);
	setSubterm(term);
	this.size = 1+term.size;
    }
    public final FodaPA reduce() {
	if (getAxStrict()) {
	    List subterms = new LinkedList<FodaPA>();
	    subterms.add(new Checkmark());
	    subterms.add(new Prefix(getFeature(), getSubterm()));
	    return new Choice(subterms);
	} else {
	    return new Checkmark();
	}
    }

    

    // public final FodaPA reduce() {
    // 	List<FodaPA> subterms = new LinkedList<FodaPA>();
    // 	subterms.add(new Prefix(this.getFeature(), this.getSubterm()));
    // 	subterms.add(new Checkmark());
    // 	numReductions++;
    // 	FodaPA newTerm = new Choice(subterms);
    // 	return newTerm;
	    
    // }
    
    // I am not sure if this is correct. It should be proven
    public final IVec<IVecInt> getClausesShort(LiteralMap idMap) {
	return 	new Vec<IVecInt>();	
    }
    // public void addClausesShort(ISolver solver, HashMap<String,Integer> idMap ) 
    // 	throws ContradictionException {
    // 	solver.addAllClauses(getClausesShort(idMap));
    // }
    
	public final IVec<IVecInt> getClausesFull(LiteralMap idMap) {
	    List<FodaPA> terms = new ArrayList<FodaPA>(2);
	    Prefix prefix = new Prefix(this.getFeature(), this.getSubterm());
	    terms.add(prefix);
	    terms.add(new Checkmark());
	    Choice choice = new Choice(terms);
	    return choice.getClauses(idMap, true);
    }
    // public void addClausesFull(ISolver solver, HashMap<String,Integer> idMap ) 
    // 	throws ContradictionException {
    // 	solver.addAllClauses(getClausesFull(idMap));
    // }

    public final IVec<IVecInt> getClauses(LiteralMap idMap,
					  boolean full) {
	if (full) {
	    return getClausesFull(idMap);
	} else {
	    return getClausesShort(idMap);
	}


    }
    // public void addClauses(ISolver solver, HashMap<String,Integer> idMap,
    // 			   boolean full) 
    // 	throws ContradictionException {
    // 	if (full) {
    // 	    addClausesFull(solver, idMap);
    // 	} else {
    // 	    addClausesShort(solver, idMap);
    // 	}
    // }
    
    public static Set<Product> denotational(String feature,
					    Set<Product> products) {
	Set<Product> res = null;
	if (getAxStrict()) {
	    for (Product p: products){
		p.add(feature);
	    }
	    res = products;
	} else {
	    res = new HashSet<Product> ();
	}
	res.add(new Product());
    	return res;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	return denotational(getFeature(), getSubterm().denotational());
    }
    
  
    public String toString() {
    	return String.format("[%1s];%2s",this.getFeature(),
			     this.getSubterm().toString());
    }

}
