/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.apache.commons.lang3.StringUtils;
import org.sat4j.core.Vec;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class MandatoryFeature here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: Parallel.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class Parallel extends FodaPA {
    public static final String NAME = "paralel";
    /**
     * Creates a new <code>MandatoryFeature</code> instance.
     *
     */
    public Parallel(List<FodaPA> terms) {
	super();
    	setSubterms(terms);
	size = 1+FodaPA.maxSize(this.getSubterms());
    }

    private static boolean allTermsCheck(List<FodaPA> terms) {
    	for (Iterator<FodaPA> itr = terms.iterator(); itr.hasNext();) {
	    FodaPA fodaPA = itr.next();
	    if (! (fodaPA instanceof Checkmark) ) {
		return false;
	    }
	}
	return true;
    }
    
    /**
     * Describe <code>reduce</code> method here.
     *
     * @return a <code>ComboFodaPABool</code> value
     */
    public final FodaPA reduce() {
	List<FodaPA> newSubterms = new LinkedList<FodaPA>();
	List<String> features = new LinkedList<String>();

	int nr=numReductions;
	
	for (FodaPA subterm: this.getSubterms()) {
	    if (subterm instanceof Choice) {
		// We do not reduce the choice, we will make the 
		// distribution later.
		newSubterms.add(subterm);
	    } else if (subterm instanceof Prefix) {
		//if it is a Prefix we do not reduce.
		Prefix pref = (Prefix) subterm;
		features.add(pref.getFeature());
		newSubterms.add(pref.getSubterm());
	    } else {
		FodaPA newTerm = subterm.reduce();
		//If we reduce by induction it is basic term.
		if (newTerm.isNil()) {
		    numReductions++;
		    return new Nil();
		}
		if (newTerm instanceof Prefix) {
		    Prefix pref = (Prefix) newTerm;
		    features.add(pref.getFeature());
		    newSubterms.add(pref.getSubterm());
		} else if ( (newTerm instanceof Checkmark) ) {
		    //We remove the checkmar, so it is a reduction
		    numReductions++;
		} else {
		    newSubterms.add(newTerm);
		} //if it is a choice we will distribute later.
	    }
	}
	// if there are no features then there the element is a parallel of choices.
	if (features.size()==0) {
	    if (newSubterms.size()==0) {
		//In this case all subterms are checkmark and we did not introduce them.
		numReductions ++;
		return new Checkmark();
	    } else if (newSubterms.size()==1) {
		numReductions++;
		return newSubterms.get(0);
	    } else {
		//all new subterms are choices of at least 2 elements
		FodaPA fpa = reduceChoices(newSubterms);
		return fpa;
	    } 
	}
	FodaPA inner = null;
	if (newSubterms.size()==0) {
	    numReductions++;
	    inner = new Checkmark();
	} else if (newSubterms.size()==1) {
	    numReductions++;
	    inner = newSubterms.get(0);
	} else {
	    inner = new Parallel(newSubterms); 
	}

	for (Iterator<String> itr = features.iterator(); itr.hasNext();) {
	    String feature  = itr.next();
	    numReductions++;
	    inner = new Prefix(feature, inner);
	}
	return inner;
    }
    
    public String toString() {
	return "("+StringUtils.join(this.getSubterms(), ") || (")+")";
    }

    public final IVec<IVecInt> getClauses(LiteralMap idMap,
					  boolean full) {
	IVec<IVecInt> vec = new Vec<IVecInt>();
	for (FodaPA term: this.getSubterms()) {
	    IVec<IVecInt> clauses = term.getClauses(idMap,full);
	    for (Iterator<IVecInt> i = clauses.iterator(); i.hasNext();) {
		vec.push(i.next());
	    }
	}
	return vec;
    }
    // public void addClauses(ISolver solver, HashMap<String,Integer> idMap,
    // 				    boolean full ) 
    // 	throws ContradictionException {
    // 	for (FodaPA term: this.subterms) {
    // 	    term.addClauses(solver,idMap,full);
    // 	}
    // }


    private static int findChoice(List<FodaPA> terms) {
	int ninSubterms=0;
	int posChoice = -1;
	int i=0;
	for (FodaPA term : terms) {
	    if (term instanceof Choice) {
		Choice cChoice = (Choice)term;
		int nSubtermsCurrent = cChoice.getSubterms().size();
		if ( posChoice==-1 || nSubtermsCurrent<ninSubterms ) {
		    posChoice = i;
		    ninSubterms = nSubtermsCurrent;
		}
	    }
	    i++;
	}
	return posChoice;
    }
    
    private static FodaPA reduceChoices(List<FodaPA> terms) {
	// log.debug(terms.size()+":"+StringUtils.join(terms,"||||"));
	List<FodaPA> newTerms = new LinkedList<FodaPA>();
	int nr = numReductions;
    	for (FodaPA term: terms) {
	    newTerms.add(term.reduce());
	}
	if (nr<numReductions) {
	    // log.debug(newTerms.size()+":"+(newTerms.size()>=1?""+newTerms.get(0)
					   // :"No"));
	    if (newTerms.size()<=1) {
		return newTerms.get(0); 
	    } else {
		return new Parallel(newTerms);
	    }
	}

	int posChoice = findChoice(terms);
	if (posChoice == -1) {
	    return null;
	}
	// log.debug(posChoice);
	newTerms = new LinkedList<FodaPA>();
	Choice choice = null;
	int i=0;
	// log.debug(i+":"+posChoice);
	for (FodaPA term : terms) {
	    if (i!=posChoice) {
		newTerms.add(term);
	    } else {
		choice = (Choice)term;
	    }
	    i++;
	}
	// log.debug(i+":"+posChoice+":"+choice);
	Parallel newParallel = new Parallel(newTerms);
	// log.debug(choice+":"+newParallel);
	substitueCheckmark(choice, newParallel);
	// log.debug(i+":"+posChoice+":"+choice);
	return choice;
    }

    private static Set<Product> denotational(Set<Product> s1,
					     Set<Product> s2) {
	Set<Product> par = new HashSet<Product>();
	for (Iterator<Product> itr1 = s1.iterator(); itr1.hasNext();) {
	    Product p1 = itr1.next();
	    for (Iterator<Product> itr2 = s2.iterator(); itr2.hasNext();) {
		Product p2 = itr2.next();
		Product p = new Product();
		p.addAll(p1);
		p.addAll(p2);
		par.add(p);
	    }
	}
	return par;
    }
    private static Set<Product> denotational(Set<Product> products,
					     List<Set<Product>> terms) {
	if (terms.size()==0) {
	    return products;
	} else {
	    Iterator<Set<Product>> itr = terms.iterator();
	    Set<Product> s1 = itr.next();
	    itr.remove();
	    Set<Product> s2 = denotational(s1, products);
	    return denotational(s2, terms);
	} 
    }
    
    public static Set<Product> denotational(List<Set<Product>> subterms) {
	Set <Product> products = denotational(new Checkmark().denotational(),
					      subterms);
	return products;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	List<Set<Product>> subterms_den = new LinkedList<Set<Product>>();
	for (FodaPA subterm: getSubterms()) {
	    subterms_den.add(subterm.denotational());
	}

    	Set <Product> products = denotational(new Checkmark().denotational(),
					      subterms_den);
	return products;
    }
}
