/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Vector;
import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class MandatoryFeature here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: Prefix.java,v 1.2 2013/05/14 09:22:28 luis Exp $
 */
public class Prefix extends FodaPA {
    public static final String NAME = "mandatory_feature";
    /**
     * Creates a new <code>Prefix</code> instance.
     *
     */
    
    public Prefix(String feature, FodaPA subterm) {
	super();
	setFeature(feature);
	setSubterm(subterm);
	this.size = 1+subterm.size;
    }

    /**
     * get <code>getClauses</code> method here.
     *
     * @param arrayList an <code>ArrayList</code> value
     * @return an <code>IVec</code> value
     */
    public final IVec<IVecInt> getClauses(LiteralMap idMap,
					  boolean full) {
	int literal = idMap.getLiteral(getFeature());
	IVec<IVecInt> clauses = this.getSubterm().getClauses(idMap,full);
	if (full) {
	    for (int i = 0; i < clauses.size(); i++) {
		IVecInt clause = clauses.get(i);
		for (int j = 0; j < clause.size(); j++) {
		    if (clause.get(j)==-literal) {
			clause.set(j, literal);
		    }
		}
	    }
	    return clauses;
	} else {
	    int [] clause = {literal};
	    IVecInt vecInt = new VecInt(clause);
	    clauses.push(vecInt);
	    return clauses;
	}
    }
    
    
 
    public FodaPA reduce() {
	if (this.getSubterm().isNil()) {
	    numReductions ++;
	    return this.getSubterm();
	}

	int nr = numReductions;
    	FodaPA newTerm = this.getSubterm().reduce();
	if (nr<numReductions) {
	    numReductions ++;
	    return new Prefix(this.getFeature(), newTerm);
	} else {
	    return this;
	}
    }

    public static Set<Product> denotational(String feature, 
					    Set<Product> products) {
	for (Product p: products) {
	    p.add(feature);
	}
	return products;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	Set<Product> s = getSubterm().denotational();
	return denotational(getFeature(), s);
    }
    
  
  
    public String toString() {
    	return String.format("%1s;%2s",this.getFeature(),this.getSubterm().toString());
    }
    
}
