/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.sat4j.core.VecInt;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;

/**
 * Describe class MandatoryFeature here.
 *
 *
 * Created: Sun Nov 25 20:09:40 2012
 *
 * @author <a href="mailto:luis@portHP.interno">Luis Fernando Llana Díaz</a>
 * @version $Id: Requires.java,v 1.2 2013/05/14 09:22:29 luis Exp $
 */
public class Requires extends FodaPA {
    public static final String NAME = "requires";
    /**
     * Creates a new <code>MandatoryFeature</code> instance.
     *
     */
    public Requires(String featPre, String featSeq, FodaPA term) {
	super();
	setFeatPre(featPre);
	setFeatSeq(featSeq);
	setSubterm(term);
	this.size = this.getSubterm().size + 1;
    }

    public boolean equals(Object other) {
	if ( !( other instanceof Requires ) ) {
	    return false;
	} 
	Requires req = (Requires) other;
	if (!this.getFeatPre().equals(req.getFeatPre()) || 
	    !this.getFeatSeq().equals(req.getFeatSeq())) {
	    return false;
	}
	return this.getSubterm().equals(req.getSubterm());
    }

    public final IVec<IVecInt> getClauses(LiteralMap idMap,
					  boolean full) {
	IVec<IVecInt> clauses = this.getSubterm().getClauses(idMap,full);
	int litPre = idMap.getLiteral(this.getFeatPre());
	int litSeq = idMap.getLiteral(this.getFeatSeq(), true);
	int [] clause = {-litPre,litSeq};
	IVecInt vecInt = new VecInt(clause);
	clauses.push(vecInt);
	return clauses;
    }
    
    // public void addClauses(ISolver solver, HashMap<String,Integer> idMap,
    // 				    boolean full) 
    // 	throws ContradictionException{
    // 	int litPre = FodaPA.getLiteral(idMap, this.getFeatPre());
    // 	int litSeq = FodaPA.getLiteral(idMap, this.getFeatSeq());
    // 	int [] clause = {-litPre,litSeq};
    // 	IVecInt vecInt = new VecInt(clause);
    // 	solver.addClause(vecInt);
    // 	this.subterm.addClauses(solver, idMap, full);
    // }

    /**
     * Describe <code>reduce</code> method here.
     *
     * @return a <code>ComboFodaPABool</code> value
     */
    public final FodaPA reduce() {
	int nr=numReductions;
	FodaPA subterm = getSubterm();
	if (subterm.isNil()) {
	    numReductions++;
	    return subterm;
	} else if (subterm instanceof Checkmark) {
	    numReductions++;
	    return subterm;
	} else if (subterm instanceof Prefix) {
	    numReductions++;
	    Prefix prefix = (Prefix)subterm;
	    if (prefix.getFeature().equals(this.getFeatPre())) {
		return new Prefix(this.getFeatPre(), 
				  new Mandatory(this.getFeatSeq(), prefix.getSubterm()));
	    } if (prefix.getFeature().equals(this.getFeatSeq())) {
		return prefix;
	    } else {
		return new Prefix(prefix.getFeature(), 
				  new Requires(this.getFeatPre(), this.getFeatSeq(), prefix.getSubterm()));
	    }
	} else if (subterm instanceof Choice) {
	    List<FodaPA> newSubterms = new LinkedList<FodaPA>();
	    for (Iterator<FodaPA> iter = ((Choice)subterm).getSubterms().iterator(); iter.hasNext();) {
		numReductions++;
		newSubterms.add(new Requires(this.getFeatPre(), this.getFeatSeq(),  iter.next()));
	    }
	    return new Choice(newSubterms);
	} else {
	    FodaPA newSubterm = subterm.reduce();
	    if (nr<numReductions) {
		return new Requires(this.getFeatPre(), this.getFeatSeq(), newSubterm);
	    } else {
		return this;
	    }
	}
    }

    public static Set<Product> denotational(String pre, String seq, 
					   Set<Product> s) {
	for (Product p: s){
	    if (p.contains(pre)) {
		p.add(seq);
	    }
	}
	return s;
    }
    /**
     * Describe <code>denotational</code> method here.
     *
     * @return a <code>Set</code> value
     */
    public final Set<Product> denotational() {
	return denotational(getFeatPre(), getFeatSeq(), 
			    getSubterm().denotational());
    }
    public String toString() {
    	return String.format("%1s => %2s in (%3s)",getFeatPre(), getFeatSeq(), getSubterm().toString());
    }
}
