/*
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 */
package fodaat;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import org.sat4j.core.Vec;
import org.sat4j.core.VecInt;
import org.sat4j.minisat.SolverFactory;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.IProblem;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVec;
import org.sat4j.specs.IVecInt;
import org.sat4j.specs.TimeoutException;

/**
 * Describe class SatSolver here.
 *
 *
 * Created: Wed Nov 28 14:35:17 2012
 *
 * @author <a href="mailto:luis@kimba.mat.ucm.es"></a>
 * @version $Id: SatSolver.java,v 1.2 2013/05/14 09:22:29 luis Exp $
 */
public class SatSolver {

    private static void printMap(LiteralMap idMap,
				 IVec<IVecInt> clauses) {
	System.out.println("Map:"+idMap.toString());
	System.out.println("clauses:-----------");
	for (int i = 0; i < clauses.size(); i++) {
	    System.out.println(clauses.get(i));
	}
	System.out.println("-----------");	
    }
    public static void main(String [] args) throws Exception {
	File file = new File(args[0]);
	boolean full = false;
	
	FodaPA fodaPA = FodaPA.readXML(file);
	System.out.println("Término:\n"+fodaPA+":"+fodaPA.getClass().getName());
	System.out.print(args[0]);
	
	Date start = new Date();
	solve(fodaPA, false);
	Date end = new Date();
	System.out.println(String.format(" %d",end.getTime()-start.getTime()));
	//System.out.println();
	// start = new Date();
	// solve(fodaPA, true);
	// end = new Date();
	// System.out.print(end.getTime()-start.getTime());
    }

    public static void solve(FodaPA fodaPA, boolean full) 
	throws TimeoutException{	
	ISolver solver = SolverFactory.newDefault();
	LiteralMap idMap = new LiteralMap();
	fodaPA.addFeatures(idMap);
	IVec<IVecInt> clauses = fodaPA.getClauses(idMap, full);
	idMap.addSequentialRestrictions(clauses);
	//printMap(idMap, clauses);
	//fodaPA.addClauses(solver, idMap, false);
	solver.newVar(idMap.size());
	try {
	    solver.addAllClauses(clauses);
	    IProblem problem = solver;
	    
	    if (problem.isSatisfiable()) {
		System.out.print(" true");
	    } else {
		System.out.print(" false");
	    }
	} catch (ContradictionException e) {
	    System.out.print(" false");
	}   
    }
    
    
}
